<?php

new Bifit();

class Bifit
{
	private static $lang = array();
	private static $pluginName = '';

	public function __construct()
	{
		self::$pluginName = PM::getFolderPlugin(__DIR__);
		self::createLocale();
	}

	private function createLocale()
	{
		// Подключение локали плагина для текущего языка	из папки /locales
		self::$lang = PM::plugLocales(self::$pluginName);

		// Подключение локали движка для текущего языка /mg-admin/locales
		include 'mg-admin/locales/' . MG::getSetting('languageLocale') . '.php';

		// Объединение локалей в один массив для дальнейшей работы
		$lang = array_merge($lang, self::$lang);

		self::$lang = $lang;
	}

	function __checkPhone($tel)
	{
		$lang = self::$lang;

		if (isset($tel) && trim($tel) != '') {
			$phone = str_replace(
				array('+', '(', ')', '-', ' '),
				'',
				trim($tel)
			);
			$first = substr($phone, "0", 1);
			$phone =
				strlen($phone) == 10 && $first != 7 ? '7' . $phone : $phone;
			$phone = !preg_match("/^[0-9]{11}+$/", $phone)
				? $lang['PHONE_NUMBER_IS_INCORRECT']
				: $phone;
		} else {
			$phone = $lang['PHONE_NUMBER_IS_EMPTY_OR_NOT_FOUND'];
		}

		return $phone;
	}

	function __checkEmail($email)
	{
		$lang = self::$lang;

		if (isset($email) && trim($email) != '') {
			$email = !preg_match("/^\+\d+$|^\S+@\S+$/", $email)
				? $lang['MAILBOX_IS_INVALID']
				: $email;
		} else {
			$email = $lang['MAILBOX_IS_EMPTY_OR_NOT_FOUND'];
		}

		return $email;
	}

	function __checkInn($inn, $type = 1)
	{
		$lang = self::$lang;

		if (isset($inn) && trim($inn) != '') {
			if ($type == 2) {
				$inn = !preg_match("/^\d{10}$/", trim($inn))
					? $lang['INCORRECT_TIN']
					: trim($inn);
			} elseif ($type == 1) {
				$inn = !preg_match("/^\d{12}$/", trim($inn))
					? $lang['INCORRECT_TIN']
					: trim($inn);
			} else {
				$inn = !preg_match("/^\d{10}$|^\d{12}$/", trim($inn))
					? $lang['INCORRECT_TIN']
					: trim($inn);
			}
		} else {
			$inn = $lang['TIN_NOT_SET'];
		}

		return $inn;
	}

	function __checkResponse($response)
	{
		if (
			is_object($response) &&
			($response->type == 'ERROR' || $response->status == 400)
		) {
			$mess = '';
			if ($response->type == 'ERROR') {
				$array = json_decode(json_encode($response), true);
				array_walk_recursive($array, function ($item, $key) use (
					&$mess
				) {
					if ($item != 'ERROR' && trim($item != '')) {
						$mess .= $item . "\n";
					}
				});
			} elseif ($response->status == 400) {
				$mess .= $response->error . "\n";
				$mess .= $response->message . "\n";
			}

			return $mess;
		}

		return false;
	}

	function __getData($url, $headers = false, $data = false, $type = 'POST')
	{
		$ch = curl_init($url);

		if ($type != 'POST') {
			curl_setopt($ch, CURLOPT_CUSTOMREQUEST, $type);
		} else {
			curl_setopt($ch, CURLOPT_POST, true);
		}

		if ($data) {
			curl_setopt($ch, CURLOPT_POSTFIELDS, $data);
		}

		curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
		curl_setopt($ch, CURLOPT_ENCODING, '');
		curl_setopt($ch, CURLOPT_MAXREDIRS, 10);
		curl_setopt($ch, CURLOPT_TIMEOUT, 0);
		curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true);
		curl_setopt($ch, CURLOPT_HTTP_VERSION, CURL_HTTP_VERSION_1_1);

		if ($headers) {
			curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
		}

		$output = curl_exec($ch);
		curl_close($ch);

		return $output;
	}

	function __getToken($login, $password)
	{
		$signature = base64_encode(hash('sha256', $password, true));
		$data = 'username=' . $login;
		$data .= '&password=' . $signature;
		$data .= '&client_id=cashdesk-rest-client';
		$data .= '&client_secret=cashdesk-rest-client';
		$data .= '&grant_type=password';
		$url = 'https://kassa.bifit.com/cashdesk-api/v1/oauth/token';
		$headers = array('Content-Type: application/x-www-form-urlencoded');
		$tokenData = json_decode(self::__getData($url, $headers, $data));

		return $tokenData;
	}

	function __getOrganizations($token)
	{
		$url =
			'https://kassa.bifit.com/cashdesk-api/v1/protected/organizations/list/read';
		$headers = array('Authorization: Bearer ' . $token);
		$organizations = json_decode(self::__getData($url, $headers));

		return $organizations;
	}

	function __getTradeObjects($org_id, $token)
	{
		$data = 'organization_id=' . $org_id;
		$url =
			'https://kassa.bifit.com/cashdesk-api/v1/protected/trade_objects/list/read';
		$headers = array(
			'Content-Type: application/x-www-form-urlencoded',
			'Authorization: Bearer ' . $token
		);
		$trade_objects = json_decode(self::__getData($url, $headers, $data));

		return $trade_objects;
	}

	function __getMainArray($token)
	{
		$arr = [];
		$orgs = self::__getOrganizations($token);

		foreach ($orgs as $key => $org) {
			$arr[$org->id]['name'] = $org->name;
			$arr[$org->id]['traides'] = [];

			$trds = self::__getTradeObjects($org->id, $token);

			foreach ($trds as $k => $trd) {
				$arr[$org->id]['traides'][$trd->id] = $trd->name;
			}
		}

		return $arr;
	}

	function __getOrgsData($options, $type = null, $id = null, $up = null)
	{
		$pluginName = self::$pluginName;
		$organizations = $options['organizations']
			? $options['organizations']
			: null;

		if (
			$options['login'] &&
			$options['password'] &&
			(!$organizations || $up)
		) {
			$data = self::__getToken(
				Bifit::__checkPhone($options['login']),
				$options['password']
			);
			$token = $data->access_token;
			$organizations = self::__getMainArray($token);
		}

		$org_id = $id ? $id : $options['organization'];

		if (!$org_id || $org_id == '') {
			$org_id = array_key_first($organizations);
		}

		$trade_objects = $organizations[$org_id]['traides'];
		$trd_id = array_key_first($trade_objects);

		if (
			!$options['organizations'] ||
			!$options['organization'] ||
			!$options['trade_object'] ||
			$up
		) {
			$options['organizations'] = $organizations;
			$options['organization'] = $org_id;
			$options['trade_object'] = $trd_id;

			MG::setOption(array(
				'option' => $pluginName . '-option',
				'value' => addslashes(serialize($options))
			));

			if ($up) {
				return true;
			}
		}

		return $type ? $trade_objects : $organizations;
	}

	function __getOrderData($order, $options)
	{
		// data
		$data = new stdClass();
		$data->id = $order['id'];
		$data->number = $order['number'];
		$data->created = $order['add_date'];
		$data->changed = $order['update_date'];
		$data->total = $order['summ'] + $order['delivery_cost'];
		$data->delivery_id = $order['delivery_id'];
		$data->delivery_cost = $order['delivery_cost'];
		$data->payment_id = $order['payment_id'];
		$data->status_id = $order['status_id'];
		$data->paid = $order['paided'];
		$data->pay_date = $order['pay_date'];
		$data->comment = $order['_comment'];
		$data->user_comment = $order['user_comment'];
		$data->user_name =
			isset($order['name_parts']) && $order['name_parts'] != ''
				? $order['name_parts']
				: $order['name_buyer'];
		$data->user_email =
			isset($order['contact_email']) && $order['contact_email'] != ''
				? self::__checkEmail($order['contact_email'])
				: null;
		$data->user_phone =
			isset($order['phone']) && $order['phone'] != ''
				? self::__checkPhone($order['phone'])
				: null;
		$data->user_address =
			isset($order['address']) && $order['address'] != ''
				? $order['address']
				: null;
		$data->address =
			$options['address_type'] == 'email'
				? $data->user_email
				: $data->user_phone;
		$data->products = unserialize(stripslashes($order['order_content']));
		$data->cashier_name = $options['cashier_name'];
		$data->cashier_inn = $options['cashier_inn'];
		$data->payment_address = $options['payment_address'];
		$data->calc_method = $options['calculation_method'];
		$data->payment_type = $options['payment_type'];
		$data->vat_mode = $options['vat_mode'];
		$data->vat = $options['vat'];

		return $data;
	}

	function __saveReport($order, $receipt_id, $doc, $bifit)
	{
		$lang = self::$lang;

		// display the property of the operation
		$log =
			$order->test != ''
				? '####### ' . $lang['TEST_REQUEST'] . "\n"
				: '####### ' . $lang['COMBAT_REQUEST'] . "\n";

		// set the time zone according to the site settings
		$log .= '####### ' . date('Y-m-d H:i:s', time()) . "\n";

		// ANSWER #1

		$log .= '####### ' . $lang['ANSWER'] . ' #1: ' . "\n";

		// response data # 1
		if (is_object($order->bifit)) {
			foreach ($order->bifit as $key => $val) {
				$log .= $key . ': ' . $val . "\n";
			}
		}

		// RECEIPT

		// generated receipt for sending
		$log .=
			'####### ' .
			$lang['RECEIPT'] .
			': ' .
			"\n" .
			$order->receipt .
			"\n";

		// ANSWER #2

		$log .= '####### ' . $lang['ANSWER'] . ' #2: ' . "\n";

		// response data #2
		if (is_object($receipt_id)) {
			$array = json_decode(json_encode($receipt_id), true);
			array_walk_recursive($array, function ($item, $key) use (&$log) {
				if ($item != 'ERROR' && trim($item) != '') {
					$log .= $item . "\n";
				}
			});
			$order->receipt_id = $lang['NOTE_SM'];
		} else {
			$order->receipt_id = $receipt_id;
			$log .= $order->receipt_id . "\n";

			// ANSWER #3

			$log .= '####### ' . $lang['ANSWER'] . ' #3: ' . "\n";
			$log .= is_object($doc) ? print_r($bifit, true) : '';

			// LINK TO THE CHECK

			// generated link to the check
			$log .=
				'####### ' .
				$lang['RECEIPT_LINK'] .
				': ' .
				"\n" .
				$order->receipt_link;
		}

		$order->log = $log;

		// SAVE IN DB

		$insert_fields = array(
			'order_id' => $order->id,
			'order_number' => $order->number,
			'order_date' => $order->created,
			'action_date' => date('Y-m-d H:i:s'),
			'receipt_link' => $order->receipt_link,
			'receipt_id' => $order->receipt_id,
			'receipt_type' => $order->receipt_type,
			'log' => $order->log
		);

		DB::buildQuery(
			"INSERT INTO `" . PREFIX . self::$pluginName . "` SET ",
			$insert_fields
		);
	}

	function __saveLogfile($data, $order_number)
	{
		$lang = self::$lang;
		$file = dirname(__FILE__) . '/../logs/log_' . $order_number . '.txt';
		$top =
			"## " .
			date('Y-m-d H:i:s') .
			" | " .
			$lang['BIFIT_CASH_DESK'] .
			" | " .
			$lang['ORDER_N'] .
			$order_number;
		$len = mb_strlen($top, "UTF-8");

		for ($i = 0; $i < $len; $i++) {
			$line .= "#";
		}

		$current = file_get_contents($file);
		$content =
			$current . $line . "\n" . $top . "\n" . $line . "\n" . $data . "\n";
		file_put_contents($file, $content);
	}

	function __saveOrder($order)
	{
		$lang = self::$lang;

		// log
		$log .= '## ' . $lang['LOG_GETTING_TOKEN'] . "\n";

		// Get token
		$data = self::__getToken($order->login, $order->password);
		$token = $data->access_token;

		// log
		if ($token) {
			$log .= '## ' . $lang['LOG_GOT_TOKEN'] . ': ' . $token . "\n";
		} else {
			$log .= '## ' . $lang['LOG_ERROR_GETTING_TOKEN'] . "\n";
			$log .=
				'## ' .
				$lang['LOG_ERROR'] .
				': ' .
				$data->error .
				"\n" .
				'## ' .
				$lang['LOG_ERROR_DESC'] .
				': ' .
				$data->error_description;

			self::__saveLogfile($log, $order->number);

			return false;
		}

		// log
		$log .= '## ' . $lang['LOG_CHECK_PRODUCTS_IN_DATABASE'] . "\n";

		// parameters for ordering
		$onlineOrderItems = '';

		// form order data
		foreach ($order->products as $key_id => $product) {
			$product = (object) $product;

			// address for POST request
			$url =
				'https://kassa.bifit.com/cashdesk-api/v1/protected/nomenclatures/external_id/' .
				$product->id .
				'?organization_id=' .
				$order->org_id;

			// link data
			$data = false;

			// headers
			$headers = array(
				'Content-Type: application/x-www-form-urlencoded',
				'Authorization: Bearer ' . $token
			);

			$prod = json_decode(self::__getData($url, $headers, $data, 'GET'));

			if ($prod == null) {
				// CREATE A NEW PRODUCT

				// log
				$log .=
					'## ' .
					$lang['LOG_PRODUCT_VS_ID'] .
					$product->id .
					' ' .
					$lang['LOG_NOT_FOUND_IN_DATABASE'] .
					"\n";
				$log .=
					'## ' . $lang['LOG_ADDING_NEW_PRODUCT_TO_DATABASE'] . "\n";

				// address for POST request
				$url =
					'https://kassa.bifit.com/cashdesk-api/v1/protected/nomenclatures';

				// link data
				$data = '{';
				$data .= '"item": {';
				$data .= '"vendorCode": "' . $product->code . '", ';
				$data .= '"name":"' . $product->name . '", ';
				$data .= '"externalId":"' . $product->id . '", ';
				$data .= '"sellingPrice":' . $product->price;
				$data .= '},';
				$data .= '"organizationId":"' . $order->org_id . '"';
				$data .= '}';

				// headers
				$headers = array(
					'Content-Type: application/json',
					'Authorization: Bearer ' . $token
				);

				// log
				$log .= $lang['LOG_DATA_TO_BE_SENT_TO_DATABASE'] . ':' . "\n";
				$log .= $data . "\n";

				// get data from request
				$tov = json_decode(self::__getData($url, $headers, $data));

				// log
				if (!is_object($tov) && !is_array($tov)) {
					$log .=
						'## ' .
						$lang['LOG_NEW_PRODUCT_ADDED_TO_DATABASE'] .
						$tov .
						"\n";
				} else {
					$log .= '## ' . $lang['LOG_ERROR_ADDING_NEW_PRODUCT'] . ':';
					$log .= self::__checkResponse($tov) . "\n";
				}
			} else {
				$tov = $prod->id;

				// log
				$log .=
					'## ' .
					$lang['LOG_PRODUCT_VS_ID'] .
					$product->id .
					' ' .
					$lang['LOG_EXISTS_IN_DATABASE_WITH_ID'] .
					$tov .
					"\n";
			}

			// display product data in the order
			$onlineOrderItems .= $onlineOrderItems != '' ? ', ' : '';
			$onlineOrderItems .= '{';
			$onlineOrderItems .= '"nomenclatureId":"' . $tov . '", ';
			$onlineOrderItems .= '"paymentSubject":"PRODUCT", ';
			$onlineOrderItems .= '"name":"' . $product->name . '", ';
			$onlineOrderItems .= '"price":' . $product->price . ', ';
			$onlineOrderItems .= '"quantity":' . $product->count;
			$onlineOrderItems .= '}';
		}

		// delivery details
		if ($order->delivery_cost > 0) {
			// address for POST request
			$url =
				'https://kassa.bifit.com/cashdesk-api/v1/protected/nomenclatures/external_id/' .
				'delivery_' .
				$order->delivery_id .
				'?organization_id=' .
				$order->org_id;

			// link data
			$data = false;

			// headers
			$headers = array(
				'Content-Type: application/x-www-form-urlencoded',
				'Authorization: Bearer ' . $token
			);

			$delivery = json_decode(
				self::__getData($url, $headers, $data, 'GET')
			);

			// log
			$log .= '## ' . $lang['LOG_CHECKING_DELIVERY_IN_DATABASE'] . "\n";

			if ($delivery == null) {
				// CREATE A NEW SERVICE

				// log
				$log .=
					'## ' . $lang['LOG_DELIVERY_NOT_FOUND_IN_DATABASE'] . "\n";
				$log .= '## ' . $lang['LOG_ADDING_DELIVERY_TO_DATABASE'] . "\n";

				// address for POST request
				$url =
					'https://kassa.bifit.com/cashdesk-api/v1/protected/nomenclatures';

				// link data
				$data = '{';
				$data .= '"item": {';
				$data .= '"name":"' . $lang['LOG_DELIVERY'] . '", ';
				$data .=
					'"externalId":"' .
					'delivery_' .
					$order->delivery_id .
					'", ';
				$data .= '"sellingPrice":' . $order->delivery_cost;
				$data .= '},';
				$data .= '"organizationId":"' . $order->org_id . '"';
				$data .= '}';

				// headers
				$headers = array(
					'Content-Type: application/json',
					'Authorization: Bearer ' . $token
				);

				// log
				$log .= $lang['LOG_DATA_TO_BE_SENT_TO_DATABASE'] . ':' . "\n";
				$log .= $data . "\n";

				// get data from request
				$tov = json_decode(self::__getData($url, $headers, $data));

				// log
				if (!is_object($tov) && !is_array($tov)) {
					$log .=
						'## ' .
						$lang['LOG_DELIVERY_ADDED_TO_DATABASE'] .
						$tov .
						"\n";
				} else {
					$log .=
						'## ' . $lang['LOG_ERROR_WHILE_ADDING_DELIVERY'] . "\n";
					$log .= self::__checkResponse($tov) . "\n";
				}
			} else {
				$tov = $delivery->id;

				// log
				$log .=
					'## ' .
					$lang['LOG_DELIVERY_ALREADY_EXISTS_IN_DATABASE'] .
					$tov .
					"\n";
			}

			// display data of delivery method in order
			$onlineOrderItems .= $onlineOrderItems != '' ? ', ' : '';
			$onlineOrderItems .= '{';
			$onlineOrderItems .= '"nomenclatureId":"' . $tov . '", ';
			$onlineOrderItems .= '"paymentSubject":"SERVICE", ';
			$onlineOrderItems .= '"name":"' . $lang['LOG_DELIVERY'] . '", ';
			$onlineOrderItems .= '"price":' . $order->delivery_cost . ', ';
			$onlineOrderItems .= '"quantity":1';
			$onlineOrderItems .= '}';
		}

		// CREATE A USER IN THE CASH

		// address for POST request
		$url = 'https://kassa.bifit.com/cashdesk-api/v1/protected/clients';

		// headers
		$headers = array(
			'Content-Type: application/json',
			'Authorization: Bearer ' . $token
		);

		// link data
		$data = '{';
		$data .= '"organizationId":"' . $order->org_id . '", ';
		$data .= '"address":"' . $order->user_address . '", ';
		$data .= '"lastName":"", ';
		$data .= '"firstName":"' . trim($order->user_name) . '", ';
		$data .= '"phone":"' . $order->user_phone . '", ';
		$data .= '"email":"' . $order->user_email . '"';
		$data .= '}';

		// log
		$log .= '## ' . $lang['LOG_TRYING_ADD_NEW_CLIENT_TO_DATABASE'] . "\n";
		$log .= '## ' . $lang['LOG_DATA_TO_BE_SENT_TO_DATABASE'] . ':' . "\n";
		$log .= $data . "\n";

		// trying to add a new client
		$client = json_decode(self::__getData($url, $headers, $data));

		// such a client is already in the database
		if (is_object($client) && $client->type == "ERROR") {
			// address for POST request
			$url =
				'https://kassa.bifit.com/cashdesk-api/v1/protected/clients/list/read?organization_id=' .
				$order->org_id;

			// link data
			$data = false;

			// headers
			$headers = array(
				'Content-Type: application/json',
				'Authorization: Bearer ' . $token
			);

			// get all clients
			$clients = json_decode(self::__getData($url, $headers, $data));

			foreach ($clients as $item) {
				// phone number match
				if ($item->phone == $order->user_phone) {
					$client = $item->id;
				}
			}

			// log
			$log .=
				'## ' .
				$lang['LOG_CLIENT_ALREADY_EXISTS_IN_DATABASE'] .
				$client .
				"\n";
		} else {
			// log
			$log .=
				'## ' .
				$lang['LOG_NEW_CLIENT_ADDED_TO_DATABASE'] .
				$client .
				"\n";
		}

		// SENDING AN ORDER TO THE CASHIER

		// checking order availability
		// address for GET request
		$url =
			'https://kassa.bifit.com/cashdesk-api/v1/protected/online_orders/list/read/' .
			$client .
			'?organization_id=' .
			$order->org_id;

		// link data
		$data = false;

		// headers
		$headers = array(
			'Content-Type: application/x-www-form-urlencoded',
			'Authorization: Bearer ' . $token
		);

		// get orders
		$orders = json_decode(self::__getData($url, $headers, $data, 'GET'));

		// params
		$order_id = null;
		$delivery_type = '';

		// log
		$log .= '## ' . $lang['LOG_CHECKING_ORDER_IN_DATABASE'] . "\n";

		// trans
		$l = [
			'COURIER' => $lang['LOG_COURIER'],
			'SELF' => $lang['LOG_SELF']
		];

		// search order by id
		foreach ($orders as $key => $item) {
			if ($item->externalId == $order->number) {
				$order_id = $item->id;
				$delivery_type = $item->deliveryType;

				// log
				$log .=
					'## ' .
					$lang['LOG_CURRENT_ORDER_ALREADY_EXIST'] .
					$order_id .
					'. ' .
					$lang['LOG_DELIVERY_TYPE'] .
					$l[$delivery_type] .
					'.' .
					"\n";

				break;
			}
		}

		// log
		if (!$order_id) {
			$log .=
				'## ' . $lang['LOG_CURRENT_ORDER_NOT_FOUND_IN_DATABASE'] . "\n";
		}

		// type of delivery
		$deliveryType = 'SERVICE';
		if (
			isset($order->courier) &&
			in_array($order->delivery_id, $order->courier)
		) {
			$deliveryType = 'COURIER';
		} elseif (
			isset($order->self) &&
			in_array($order->delivery_id, $order->self)
		) {
			$deliveryType = 'SELF';
		}

		// assign "is_update"
		$is_update =
			$order_id > 0 &&
			isset($order->refused_statuses) &&
			in_array($order->status_id, $order->refused_statuses) &&
			$order->use_refused
				? 1
				: null;

		// new order || "is_update" conditions
		if ($deliveryType != 'SERVICE' && ($order_id == 0 || $is_update)) {
			// address for request
			$url =
				'https://kassa.bifit.com/cashdesk-api/v1/protected/online_orders';

			// headers
			$headers = array(
				'Content-Type: application/json',
				'Authorization: Bearer ' . $token
			);

			// data
			if ($is_update) {
				// log
				$log .=
					'## ' .
					$lang['LOG_BEGIN_UPDATING_ORDER_IN_DATABASE'] .
					"\n";

				$url .= '/' . $order_id . '/status';
				$type = 'PUT';
				$statusType = $delivery_type == 'SELF' ? 'REFUSED' : 'DELETED';
				// Basic data
				$data = '{';
				$data .= '"onlineOrderId":"' . $order_id . '",';
				$data .= '"statusType":"' . $statusType . '",';
				$data .= '"statusDate":0,';
				$data .= '"statusInfo":""';
				$data .= '}';
			} else {
				// log
				$log .=
					'## ' . $lang['LOG_BEGIN_SAVING_ORDER_IN_DATABASE'] . "\n";

				// data
				$type = 'POST';
				$statusType =
					$deliveryType == 'SELF'
						? 'ACCEPTED'
						: strtoupper($order->new_status);

				// timestump
				$ts = strtotime($order->created);
				$orderTime = strlen($ts) == 10 ? $ts * 1000 : $ts;

				// Basic data
				$onlineOrder = '{';
				$onlineOrder .= '"organizationId":"' . $order->org_id . '", ';
				$onlineOrder .= '"tradeObjectId":"' . $order->trade_id . '", ';
				$onlineOrder .=
					'"taxSystem":"' . strtoupper($order->tax_system) . '", ';
				$onlineOrder .= '"externalId":"' . $order->number . '", ';
				$onlineOrder .= '"deliveryType":"' . $deliveryType . '", ';
				$onlineOrder .=
					'"deliveryAmount":"' . $order->delivery_cost . '", ';
				$onlineOrder .= '"paid":false, ';
				$onlineOrder .= '"orderTime":' . $orderTime . ', ';
				$onlineOrder .= '"clientId":' . $client . ', ';
				$onlineOrder .= '"address":"' . $order->user_address . '", ';
				$onlineOrder .=
					'"comment":"' .
					trim($order->user_comment . ' ' . $order->comment) .
					'", ';
				$onlineOrder .=
					'"responsiblePersonLogin":"' . $order->login . '", ';
				$onlineOrder .= '"totalAmount":' . $order->total . ', ';
				$onlineOrder .= '"currentStatusType":"' . $statusType . '", ';
				$onlineOrder .= '"currentStatusTime":0';
				$onlineOrder .= '}';

				$data = '{';
				$data .= '"onlineOrder":' . $onlineOrder . ', ';
				$data .= '"onlineOrderItems":[' . $onlineOrderItems . ']';
				$data .= '}';
			}

			// log
			$log .=
				'## ' . $lang['LOG_DATA_TO_BE_SENT_TO_DATABASE'] . ':' . "\n";
			$log .= $data . "\n";

			// get the ID of the document being processed (id)
			$response = json_decode(
				self::__getData($url, $headers, $data, $type)
			);

			// log
			if (!is_object($response) && !is_array($response)) {
				if ($is_update && $statusType == 'DELETED') {
					$log .=
						'## ' .
						$lang['LOG_ORDER_DELETED_FROM_DATABASE'] .
						$response .
						"\n";
				} elseif ($is_update && $statusType == 'REFUSED') {
					$log .=
						'## ' .
						$lang['LOG_ORDER_CANCELED_IN_DATABASE'] .
						$response .
						"\n";
				} else {
					$log .=
						'## ' .
						$lang['LOG_ORDER_SAVED_TO_DATABASE'] .
						$response .
						"\n";
				}
			} else {
				$log .=
					'## ' . $lang['LOG_ERROR_WHILE_SAVING_ORDER'] . ':' . "\n";
				$log .= self::__checkResponse($response) . "\n";
			}
		} else {
			// log
			if ($order_id == 0) {
				$txt = $lang['LOG_FAILED_SAVE_CURRENT_ORDER'];
			} elseif ($is_update) {
				$txt = $lang['LOG_FAILED_UPDATE_CURRENT_ORDER'];
			}

			if ($txt != '' && $deliveryType == 'SERVICE') {
				$txt .= ' ';
				$txt .= $lang['LOG_DELIVERY_NOT_COURIER_OR_SELF'];
			}

			$log .= $txt != '' ? '## ' . $txt . "\n" : '';
		}

		// send payment information
		if (
			$order->paid &&
			in_array(strtoupper($order->calc_method), [
				'PREPAY_FULL',
				'PREPAY_PARTIAL',
				'FULL_PAY'
			])
		) {
			// log
			$log .= '## ' . $lang['LOG_BEGIN_TRANSFERRING_PAYMENT_DATA'] . "\n";

			// address for request
			$url =
				'https://kassa.bifit.com/cashdesk-api/v1/protected/online_orders/payments';

			// headers
			$headers = array(
				'Content-Type: application/json',
				'Authorization: Bearer ' . $token
			);

			// data
			$type = 'POST';
			$data = '{';
			$data .= '"onlineOrderExternalId":"' . $order->number . '",';
			$data .= '"organizationId":"' . $order->org_id . '", ';
			$data .=
				'"calculationMethod":"' .
				strtoupper($order->calc_method) .
				'", ';
			$data .= '"total": ' . $order->total;
			$data .= '}';

			// log
			$log .=
				'## ' . $lang['LOG_DATA_TO_BE_SENT_TO_DATABASE'] . ':' . "\n";
			$log .= $data . "\n";

			// get the ID of the document being processed (id)
			$response = json_decode(
				self::__getData($url, $headers, $data, $type)
			);

			// log
			if (!is_object($response) && !is_array($response)) {
				$log .=
					'## ' .
					$lang['LOG_TRANSFER_OF_PAYMENT_DATA_WAS_SUCCESSFUL'] .
					$response .
					"\n";
			} else {
				$log .=
					'## ' .
					$lang['LOG_ERROR_WHILE_TRANSMITTING_PAYMENT_DATA'] .
					':' .
					"\n";
				$log .= self::__checkResponse($response) . "\n";
			}
		}

		// save log
		self::__saveLogfile($log, $order->number);
	}

	function __doFiscalize($order)
	{
		$lang = self::$lang;

		// 1. AUTHORIZATION
		//    process necessary to connect a client to fiscal processing

		// address for POST request
		$url =
			'https://fp' .
			$order->test .
			'.bifit.com/processing-api/oauth/token';

		// link data
		$data = 'token=' . $order->token;
		$data .= '&client_id=processing-connector-token';
		$data .= '&client_secret=processing-connector-token';
		$data .= '&grant_type=token';

		// headers
		$headers = array('Content-Type: application/x-www-form-urlencoded');

		// decode the received json response
		$bifit = json_decode(self::__getData($url, $headers, $data));

		// 2. FORMATION OF THE CONTENT OF A CHECK
		//    formation of information for further sending for fiscalization

		// check parameters
		$items = '';

		// generate check data
		foreach ($order->products as $key_id => $product) {
			$product = (object) $product;
			// display the product data in the check
			$items .= $items != '' ? ', ' : '';
			$items .= '{';
			$items .=
				'"calculationMethod":"' .
				strtoupper($order->calc_method) .
				'", ';
			$items .= '"paymentSubject":"PRODUCT", ';
			$items .= '"name":"' . $product->name . '", ';
			$items .= '"price":' . $product->price . ', ';
			$items .= '"quantity":' . $product->count . ', ';
			$items .=
				$vat_mode == 1
					? '"vat":"' . strtoupper($order->vat) . '", '
					: '"vat":"WITHOUT_VAT", ';
			$items .= '"total":' . (float) ($product->price * $product->count);
			$items .= '}';
		}

		// delivery
		if ($order->delivery_cost > 0) {
			// display the data of the delivery method in the check
			$items .= $items != '' ? ', ' : '';
			$items .= '{';
			$items .=
				'"calculationMethod":"' .
				strtoupper($order->calc_method) .
				'", ';
			$items .= '"paymentSubject":"SERVICE", ';
			$items .= '"price":' . $order->delivery_cost . ', ';
			$items .= '"quantity":1, ';
			$items .=
				$order->vat_mode == 1
					? '"vat":"' . strtoupper($order->vat) . '", '
					: '"vat":"WITHOUT_VAT", ';
			$items .= '"total":' . $order->delivery_cost;
			$items .= '}';
		}

		// form a check
		$receipt = '{';
		$receipt .=
			'"type":"' .
			strtoupper($order->receipt_type) .
			'", "taxSystem":"' .
			strtoupper($order->tax_system) .
			'", ';
		$receipt .= '"cashier":{';
		$receipt .= '"name":"' . $order->cashier_name . '", ';
		$receipt .= '"inn":"' . self::__checkInn($order->cashier_inn, 1) . '"';
		$receipt .= '}, ';
		$receipt .= '"client":{';
		$receipt .= '"address":"' . $order->address . '"';
		$receipt .= '}, ';
		$receipt .= '"paymentAddress":"' . $order->payment_address . '", ';
		$receipt .= '"items":[' . $items . '], ';
		$receipt .= '"total":' . $order->total . ', ';
		$receipt .=
			'"payments":{"' .
			strtoupper($order->payment_type) .
			'": ' .
			$order->total .
			'}';
		$receipt .= '}';

		// 3. SENDING A CHECK TO FISCAL PROCESSING
		//    transfer of the content of the check to fiscal processing for further fiscalization

		// address for POST request
		$url =
			'https://fp' .
			$order->test .
			'.bifit.com/processing-api/protected/documents/registration/receipts';

		// check data
		$data = $receipt;

		// Idempotency-Key
		$k = $order->number;
		$e = $order->total;
		$y = date('Y-m-d H:i:s');
		$idemp_key = hash('sha256', "$k/$e/$y");

		// headers
		$headers = array(
			'Content-Type: application/json',
			'Authorization: Bearer ' . $bifit->access_token,
			'Idempotency-Key: ' . $idemp_key
		);

		// get the ID of the document being processed (id)
		$receipt_id = json_decode(self::__getData($url, $headers, $data));

		// 4. RECEIVING DOCUMENT BY ID
		//    obtaining a document with fiscal characteristics by its number

		// options
		$doc = '';

		if (!is_object($receipt_id)) {
			// link for GET request
			$url =
				'https://fp' .
				$test .
				'.bifit.com/processing-api/protected/documents/' .
				$receipt_id;

			// headers
			$headers = array('Authorization: Bearer ' . $bifit->access_token);

			// we get the document
			$doc = json_decode(self::__getData($url, $headers, false));
		}

		// LINK TO THE CHECK
		// we form a link to the check
		$receipt_link = !is_object($receipt_id)
			? 'https://fp' .
				$order->test .
				'.bifit.com/processing-api/receipts/' .
				$idemp_key
			: 'XXX';

		// OPERATION HISTORY
		// update data
		$order->receipt = $receipt;
		$order->receipt_link = $receipt_link;
		//$order->receipt_id = !is_object($receipt_id) ? $receipt_id : '0';

		// save the operation to history
		self::__saveReport($order, $receipt_id, $doc, $bifit);
	}
}