/* 
 * Модуль BifitModule, подключается на странице настроек плагина. Скрипт будет запущен только тогда, когда страница админки будет готова к выводу настроек плагина.
 */

var BifitModule = (function() {

	return {
		lang: [], // локаль плагина 
		pluginName: "bifitkassa", // название плагина

		// Функция инициализации, выполняемая, когда страница админки будет готова вывести настройки плагина
		init: function() {

			// установка локали плагина 
			admin.ajaxRequest({
					mguniqueurl: "action/seLocalesToPlug", // действие для выполнения на сервере
					pluginName: BifitModule.pluginName
				},
				function(response) {
					BifitModule.lang = response.data
				}
			)

			// [Клик по кнопке "Смотреть" в столбце "Лог"]: выводит модальное окно для просмотра лога
			$('.admin-center').on('click', '.section-' + BifitModule.pluginName + ' .view-log', function() {
				// Определяем id окна, которое будем открывать
				var id = $(this).data('id')
				// Открываем модалку
				admin.openModal($('#plug-modal-' + id))
			})

			// [Клик по кнопке "Сбросить" в панели фильтров]
			$('.admin-center').on('click', '.section-' + BifitModule.pluginName + ' .refreshFilter', function() {
				// Перезагружаем страницу настроек плагина с параметром сброса фильтров
				admin.show(BifitModule.pluginName, "plugin", "refreshFilter=1", BifitModule.callbackBackRing)
				return false
			})

			// [Выбор в списке кол. записей на страницу]: устанавливает количиство выводимых записей в этом разделе.
			$('.admin-center').on('change', '.section-' + BifitModule.pluginName + ' .countPrintRowsEntity', function() {
				// Определяем выбранное количество записей
				var count = $(this).val()
				// Выполняем запрос в Pactioner плагина к методу setCountPrintRowsEnity()
				admin.ajaxRequest({
						mguniqueurl: "action/setCountPrintRowsEnity", // действие для выполнения на сервере
						pluginHandler: BifitModule.pluginName, // плагин для обработки запроса
						// входные данные:
						option: 'countPrintRows',
						count: count
					},
					function(response) {
						// Выводим нотификацию с результатом (ошибка или успех) 
						admin.indication(response.status, response.msg)
						// Если успех, обновляем страницу настроек плагина       
						if (response.status == "success") {
							admin.refreshPanel()
						}
					}
				)
			})			

			// [Клик по кнопке удаления в строке записи]: удаление записи
			$('.admin-center').on('click', '.section-' + BifitModule.pluginName + ' .delete-row', function() {
				// Определяем id записи, которую собираемся удалить
				var id = $(this).data('id');
				// Выполняем запрос в Pactioner плагина к методу deleteEntity()
				admin.ajaxRequest({
					mguniqueurl: "action/deleteEntity", // действие для выполнения на сервере
					pluginHandler: BifitModule.pluginName, // плагин для обработки запроса
					// входные данные:
					id: id
				}, function(response) {
					// Выводим нотификацию с результатом (ошибка или успех)
					admin.indication(response.status, response.msg)
					// Если успех, обновляем страницу настроек плагина  
					if (response.status == "success") {
						admin.refreshPanel()
					}
				})
			})

			// [Клик по кнопке обновления организаций и объектов]: загрузка списков
			$('.admin-center').on('click', '.section-' + BifitModule.pluginName + ' .update-orgs', function() {
				// Выполняем запрос в Pactioner плагина к методу deleteEntity()
				admin.ajaxRequest({
					mguniqueurl: "action/updateOrganizations", // действие для выполнения на сервере
					pluginHandler: BifitModule.pluginName // плагин для обработки запроса
				}, function(response) {
					// Выводим нотификацию с результатом (ошибка или успех) 
					admin.indication(response.status, response.msg)
					// Если успех, обновляем страницу настроек плагина  
					if (response.status == "success") {
						admin.refreshPanel()
					}
				})
			})

			// [Клик по кнопке "Сохранить" в панели настроек плагина]: сохраняет настроки плагина
			$('.admin-center').on('click', '.section-' + BifitModule.pluginName + ' .base-setting-save', function() {
				// Собираем все значения формы в один массив
				var data = {
					test: $('.section-' + BifitModule.pluginName + ' .base-setting .list-option select[name="test"]').val(),
					token: $('.section-' + BifitModule.pluginName + ' .base-setting .list-option input[name="token"]').val(),
					cashier_name: $('.section-' + BifitModule.pluginName + ' .base-setting .list-option input[name="cashier_name"]').val(),
					cashier_inn: $('.section-' + BifitModule.pluginName + ' .base-setting .list-option input[name="cashier_inn"]').val(),
					payment_address: $('.section-' + BifitModule.pluginName + ' .base-setting .list-option input[name="payment_address"]').val(),
					address_type: $('.section-' + BifitModule.pluginName + ' .base-setting .list-option select[name="address_type"]').val(),
					calculation_method: $('.section-' + BifitModule.pluginName + ' .base-setting .list-option select[name="calculation_method"]').val(),
					payment_type: $('.section-' + BifitModule.pluginName + ' .base-setting .list-option select[name="payment_type"]').val(),
					vat_mode: $('.section-' + BifitModule.pluginName + ' .base-setting .list-option select[name="vat_mode"]').val(),
					vat: $('.section-' + BifitModule.pluginName + ' .base-setting .list-option select[name="vat"]').val(),
					use_type_1: $('.section-' + BifitModule.pluginName + ' .base-setting .list-option select[name="use_type_1"]').val(),
					order_paid_1: $('.section-' + BifitModule.pluginName + ' .base-setting .list-option select[name="order_paid_1"]').val(),
					tax_system_1: $('.section-' + BifitModule.pluginName + ' .base-setting .list-option select[name="tax_system_1"]').val(),
					pm_statuses_1: $('.section-' + BifitModule.pluginName + ' .base-setting .list-option select[name="pm_statuses_1[]"]').val(),
					pm_ids_1: $('.section-' + BifitModule.pluginName + ' .base-setting .list-option select[name="pm_ids_1[]"]').val(),
					use_type_2: $('.section-' + BifitModule.pluginName + ' .base-setting .list-option select[name="use_type_2"]').val(),
					order_paid_2: $('.section-' + BifitModule.pluginName + ' .base-setting .list-option select[name="order_paid_2"]').val(),
					tax_system_2: $('.section-' + BifitModule.pluginName + ' .base-setting .list-option select[name="tax_system_2"]').val(),
					pm_statuses_2: $('.section-' + BifitModule.pluginName + ' .base-setting .list-option select[name="pm_statuses_2[]"]').val(),
					pm_ids_2: $('.section-' + BifitModule.pluginName + ' .base-setting .list-option select[name="pm_ids_2[]"]').val(),
					use_type_3: $('.section-' + BifitModule.pluginName + ' .base-setting .list-option select[name="use_type_3"]').val(),
					order_paid_3: $('.section-' + BifitModule.pluginName + ' .base-setting .list-option select[name="order_paid_3"]').val(),
					tax_system_3: $('.section-' + BifitModule.pluginName + ' .base-setting .list-option select[name="tax_system_3"]').val(),
					pm_statuses_3: $('.section-' + BifitModule.pluginName + ' .base-setting .list-option select[name="pm_statuses_3[]"]').val(),
					pm_ids_3: $('.section-' + BifitModule.pluginName + ' .base-setting .list-option select[name="pm_ids_3[]"]').val(),					
					use_type_4: $('.section-' + BifitModule.pluginName + ' .base-setting .list-option select[name="use_type_4"]').val(),
					order_paid_4: $('.section-' + BifitModule.pluginName + ' .base-setting .list-option select[name="order_paid_4"]').val(),
					tax_system_4: $('.section-' + BifitModule.pluginName + ' .base-setting .list-option select[name="tax_system_4"]').val(),
					pm_statuses_4: $('.section-' + BifitModule.pluginName + ' .base-setting .list-option select[name="pm_statuses_4[]"]').val(),
					pm_ids_4: $('.section-' + BifitModule.pluginName + ' .base-setting .list-option select[name="pm_ids_4[]"]').val(),
					use_type_5: $('.section-' + BifitModule.pluginName + ' .base-setting .list-option select[name="use_type_5"]').val(),
					order_paid_5: $('.section-' + BifitModule.pluginName + ' .base-setting .list-option select[name="order_paid_5"]').val(),
					tax_system_5: $('.section-' + BifitModule.pluginName + ' .base-setting .list-option select[name="tax_system_5"]').val(),
					pm_statuses_5: $('.section-' + BifitModule.pluginName + ' .base-setting .list-option select[name="pm_statuses_5[]"]').val(),
					pm_ids_5: $('.section-' + BifitModule.pluginName + ' .base-setting .list-option select[name="pm_ids_5[]"]').val(),
					login: $('.section-' + BifitModule.pluginName + ' .base-setting .list-option input[name="login"]').val(),
					password: $('.section-' + BifitModule.pluginName + ' .base-setting .list-option input[name="password"]').val(),
					organization: $('.section-' + BifitModule.pluginName + ' .base-setting .list-option select[name="organization"]').val(),
					trade_object: $('.section-' + BifitModule.pluginName + ' .base-setting .list-option select[name="trade_object"]').val(),
					lk_courier_delivery: $('.section-' + BifitModule.pluginName + ' .base-setting .list-option select[name="lk_courier_delivery[]"]').val(),
					lk_self_delivery: $('.section-' + BifitModule.pluginName + ' .base-setting .list-option select[name="lk_self_delivery[]"]').val(),
					lk_new_courier_order_status: $('.section-' + BifitModule.pluginName + ' .base-setting .list-option select[name="lk_new_courier_order_status"]').val(),
					lk_use_refused: $('.section-' + BifitModule.pluginName + ' .base-setting .list-option select[name="lk_use_refused"]').val(),
					lk_refused_statuses: $('.section-' + BifitModule.pluginName + ' .base-setting .list-option select[name="lk_refused_statuses[]"]').val()
				}
				// Выполняем запрос в Pactioner плагина к методу saveBaseOption()
				admin.ajaxRequest({
					mguniqueurl: "action/saveBaseOption", // действие для выполнения на сервере
					pluginHandler: BifitModule.pluginName, // плагин для обработки запроса
					// входные данные:
					data: data
				}, function(response) {
					console.log(data);
					// Выводим нотификацию с результатом (ошибка или успех)
					admin.indication(response.status, response.msg);
					// Если успех, обновляем страницу настроек плагина  
					if (response.status == "success") {
						admin.refreshPanel()
					}
				})
			})

			// [Клик по кнопке "Фильтровать" в панели фильтров]: применение выбраных фильтров
			$('.admin-center').on('click', '.section-' + BifitModule.pluginName + ' .filter-now', function() {
				BifitModule.getProductByFilter()
				return false
			})

			// [Клик по кнопке "Настройки"]: показывает/скрывает панель с настройками
			$('.admin-center').on('click', '.section-' + BifitModule.pluginName + ' .show-property-order', function() {
				// Показать/скрыть панель с настройками
				$('.property-order-container').slideToggle(function() {
					// Спрятать панель с фильтрами, если она была открыта
					$('.filter-container').slideUp()
				})
			})

			// [Клик по кнопке "Фильтры"]: показывает/скрывает панель с фильтрами
			$('.admin-center').on('click', '.section-' + BifitModule.pluginName + '  .show-filters', function() {
				// Показать/скрыть панель с фильтрами
				$('.filter-container').slideToggle(function() {
					// Спрятать панель с настройками, если она была открыта
					$('.property-order-container').slideUp()
				})
			})

			// [Выбор организации в списке]: устанавливает связные торговые объекты.
			$('.admin-center').on('change', '.section-' + BifitModule.pluginName + ' #organization', function() {
				// Определяем выбранное количество записей
				var org_id = $(this).val()
				// Выполняем запрос в Pactioner плагина к методу setCountPrintRowsEnity()
				admin.ajaxRequest({
					mguniqueurl: "action/setNewOrganization", // действие для выполнения на сервере
					pluginHandler: BifitModule.pluginName, // плагин для обработки запроса
					// входные данные:
					option: 'organization',
					org_id: org_id
				},
				function(response) {
					// Выводим нотификацию с результатом (ошибка или успех) 
					admin.indication(response.status, response.msg)
					// Если успех, обновляем страницу настроек плагина       
					if (response.status == "success") {
						admin.refreshPanel()
					}
				})
			})

			// Изменение значений в списках выбора статусов заказа
			var a = ["pm_statuses_5[]", "lk_refused_statuses[]"];
			$(a).each(function(e, t) {
				BifitModule.checkStatusSelects($('select[name="' + t + '"]'), a[0], a[1])
			}),
			$('select[name="' + a[0] + '"], select[name="' + a[1] + '"]').change(function() {
				BifitModule.checkStatusSelects($(this), a[0], a[1])
			});
			
			// Изменение значений в списках выбора способов доставки
			var n = ["lk_courier_delivery[]", "lk_self_delivery[]"];
			$(n).each(function(e, t) {
				BifitModule.checkStatusSelects($('select[name="' + t + '"]'), n[0], n[1])
			}),
			$('select[name="' + n[0] + '"], select[name="' + n[1] + '"]').change(function() {
				BifitModule.checkStatusSelects($(this), n[0], n[1])
			})
		},

		/**
		 * функция для применения фильтра
		 */
		getProductByFilter: function() {
			// Преобразуем форму с значениями фильтра в запрос
			var request = $("form[name=filter]").formSerialize()
			// Применяем фильтр 
			admin.show(BifitModule.pluginName, "plugin", request + '&applyFilter=1', BifitModule.callbackBlanck)

			return false
		},

		/**
		 *Пакет выполняемых действий после загрузки раздела товаров
		 */
		callbackBlank: function() {
			admin.AJAXCALLBACK = [{
				callback: 'admin.sortable',
				param: ['.entity-table-tbody', BifitModule.pluginName]
			}, ]
		},

		/**
		 * функция для дизактивации опций при выборе
		 */
		checkStatusSelects: function(e, t, a) {
			var n = e.attr("name") == t ? a : t;
			$.each(e.find("option"), function(e, t) {
				var a = !!$(t).is(":selected") || null;
				$('select[name="' + n + '"]').find("option").eq(e).attr("disabled", a)
			})
		}
	}
})();

// Инициализация скрипта
BifitModule.init();